@@ -3,13 +3,13 @@
     stop("This version for R 1.6 or later")
   }
   if (!frexpTest(FALSE)) {
-  	warning("frexp: failed self-test");
+  	warning("frexp: failed self-test")
   }
   if (!perturbTest()) {
-  	warning("perturb: failed self-test");
+  	warning("perturb: failed self-test")
   } 
   if (!secholTest()) {
-  	warning("sechol: failed self-test");
+  	warning("sechol: failed self-test")
   }
-  return(TRUE);
+  return(TRUE)
 }
